/*=========================================================================
 * CallNauty_sparse.c - mex file which calls Nauty using the sparse graph 
 * format.
 * Needs to be linked with the following files:
 *
 * nauty.c
 * nautil.c
 * nausparse.c
 *
 * Program Description:
 * Returns automorphism group generators (S), stabilized vertices (vstab), 
 * group size (Gsize), orbits (orb), canonical labeling (lab), and 
 * canonical version of a graph A represented by the sparse graph array 
 * inputs d, v, and e, and optional coloring matrix c. All input and output
 * arrays of this function are type double.
 *
 * Inputs:
 *      d: 1 by n array such that d(i) represents the degree (out-degree 
 *         for a digraph) of each vertex i = 1,2,...,n.
 *      v: 1 by n array such that v(i) is an index into array e such that
 *         e(v(i)),e(v(i)+1),...,e(v(i)+d(i)-1) are the vertices to which
 *         vertex i is connected.
 *      e: 1 by q array, with q being the number of edges, listing the 
 *         sets of vertices to which each vertex i is connected as
 *         described above.
 *      c: 2 by n matrix. The first row lists the vertices in some order 
 *         such that vertices of the same color are continuous. The second 
 *         row splits the listed vertices into cells corresponding to 
 *         different color classes. The ends of the color classes are 
 *         indicated by zeros.
 *
 * Outputs:
 *      S: m by n matrix of permutations of the set {1,...,n} corresponding
 *         to generating permutations of the automorphism group of A.
 *      vstab: 1 by m vector listing the stabilized vertex associated with 
 *             each generator in S (See Nauty documentation, section 6, for
 *             a more detailed explanation). The unique elements in vstab 
 *             form a base for the group generated by S and, along with S, 
 *             may be used to identify Schreier generators for the 
 *             point-wise stabilizers of elements in the base.
 *      Gsize: scalar value which is the size of the group generated by S.
 *      orb: 1 by n array containing the orbits of the automorphism 
 *           group. orb(i) is the number of the least numbered vertex in
 *           the same orbit as i.
 *      lab: 1 by n array listing the vertices in the order they need to be
 *           relabelled to give the canonical version of A.
 *      dcanon: 1 by n array representing the degree of each relabeled
 *              vertex in the canonical version of A.
 *      vcanon: 1 by n array such that v(i) is an index into ecanon such
 *              that ecanon(v(i)),ecanon(v(i)+1),...,ecanon(v(i)+d(i)-1) 
 *              are the vertices to which each relabeled vertex i is
 *              connected.
 *      ecanon: 1 by q array listing the sets of relabeled vertices to
 *              which each relabeled vertex i is connected.
 * 
 * The calling syntax is:
 *
 * [S,vstab,Gsize,orb,lab,dcanon,vcanon,ecanon] = CallNauty_sparse(d,v,e,c)
 *
 * See also, CallNauty_sparse.c, CallNauty_quick.c
 *             
 * Author:   Scott Bugenhagen
 * Contact:  sbugenha@mcw.edu
 * Date:     8/30/2011
 *
 * References:
 * 1. Brendan D. McKay. Nauty user's guide (version 2.4). Australian 
 *    National University, 2009. retrieved from 
 *    <http://cs.anu.edu.au/~bdm/nauty/> on 6/1/2011.
 *=======================================================================*/

#include <math.h>
#include "nausparse.h"
#include "mex.h"

int *generators; /* global pointer to generators */
int *stabverts; /* global pointer to stabilized vertices */

/*=======================================================================*/
/* The gateway routine */

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
  sparsegraph g, canong;
  int *lab, *ptn, *orbits;
  static DEFAULTOPTIONS_SPARSEGRAPH(options);
  statsblk stats;
  setword *workspace;
  int dmrows,vmrows,emrows,cmrows;
  int dncols,vncols,encols,cncols;
  int m,nv,ne,i,j,numgens;
  double Gsize;
  double *output[7];
  void GetGenerators(int, permutation *, int *, int, int, int);
  int setcmp(void *, int);
  
  generators = NULL;
  stabverts = NULL;
  
  /* note: all the options fields below are fixed in this program with the
   * exception of the defaultptn field. If a coloring matrix is supplied as
   * input, defaultptn is set to FALSE. If no coloring is given, it remains
   * as TRUE. Beware that the canonical labeling may differ depending on 
   * the setting of this option. Note that though digraph is always set to 
   * TRUE, the program can still handle simple graphs (though performance 
   * may be slightly worse than if the setting was FALSE). The invariant
   * "adjacencies_sg" is applied as recommended in the following link
   * http://dcsmail.anu.edu.au/pipermail/nauty-list/2010-May/000556.html
   * for digraphs. */
  
  options.cartesian = TRUE;
  options.digraph = TRUE;
  options.getcanon = TRUE;
  options.userautomproc = GetGenerators;
  options.defaultptn = TRUE;
  options.invarproc = adjacencies_sg;
  options.mininvarlevel = 0;
  options.maxinvarlevel = 99;
  
  /* check input/output for proper number of arguments */
  if(nrhs > 4)
    mexErrMsgTxt("Too many input arguments.");
  if(nrhs < 3)
    mexErrMsgTxt("Too few input arguments.");
  if(nlhs > 8) 
    mexErrMsgTxt("Too many output arguments.");
  
  /* check format of sparse graph arrays */
  if( !mxIsNumeric(prhs[0]) || !mxIsNumeric(prhs[1]) || !mxIsNumeric(prhs[2]) 
      || mxIsComplex(prhs[0]) || mxIsComplex(prhs[1]) || mxIsComplex(prhs[2]) )
     mexErrMsgTxt("Sparse graph arrays must be noncomplex numeric.");
  dmrows = mxGetM(prhs[0]);
  vmrows = mxGetM(prhs[1]);
  emrows = mxGetM(prhs[2]);
  dncols = mxGetN(prhs[0]);
  vncols = mxGetN(prhs[1]);
  encols = mxGetN(prhs[2]);
  if (dmrows != 1 || vmrows != 1 || emrows != 1)
    mexErrMsgTxt("Sparse graph arrays must be row vectors.");
  if (dncols != vncols)
    mexErrMsgTxt("Lengths of arrays d and v must be equal.");
  
  /* check format of coloring matrix if given */
  if (nrhs == 4)
  {
      if( !mxIsNumeric(prhs[3]) || mxIsComplex(prhs[3]) )
        mexErrMsgTxt("Coloring matrix must be noncomplex numeric.");
      cmrows = mxGetM(prhs[3]);
      cncols = mxGetN(prhs[3]);
      if( cmrows != 2 )
        mexErrMsgTxt("Coloring matrix must contain 2 rows.");
      if( cncols != vncols )
        mexErrMsgTxt("Coloring matrix column lengths must equal number of vertices.");
      
      options.defaultptn = FALSE;
  }
  
  /* setup sparse graph parameters */
  nv = vncols; /* number of vertices */
  ne = encols; /* number of edges */
  g.nv = nv; canong.nv = nv;
  g.nde = ne; canong.nde = ne;
  g.dlen = nv; canong.dlen = nv;
  g.vlen = nv; canong.vlen = nv;
  g.elen = ne; canong.elen = ne;
  g.wlen = 0; canong.wlen = 0;
  g.w = NULL; canong.w = NULL;
  
  /* allocate space for g, canong, lab, ptn, orbits, and workspace */
  m = (int) ceil((double)nv/(double)WORDSIZE);
  g.d = (int *) mxMalloc(nv*sizeof(int));
  g.v = (int *) mxMalloc(nv*sizeof(int));
  g.e = (int *) mxMalloc(ne*sizeof(int));
  canong.d = (int *) mxMalloc(nv*sizeof(int));
  canong.v = (int *) mxMalloc(nv*sizeof(int));
  canong.e = (int *) mxMalloc(ne*sizeof(int));
  lab = (int *) mxMalloc(nv*sizeof(int));
  ptn = (int *) mxMalloc(nv*sizeof(int));
  orbits = (int *) mxMalloc(nv*sizeof(int));
  workspace = (setword *) mxMalloc(5*m*sizeof(setword));
  if ( g.d == NULL || g.v == NULL || g.e == NULL || canong.d == NULL || 
      canong.v == NULL || canong.e == NULL || lab == NULL || ptn == NULL ||
      orbits == NULL || workspace == NULL )
    mexErrMsgTxt("mxMalloc failed.");
  
  /* unpack inputs into Nauty sparsegraphs */  
  switch(mxGetClassID(prhs[0]))
  {
      case mxDOUBLE_CLASS : {
          double *input;
          input = (double *) mxGetData(prhs[0]);
          for (j = 0; j < nv; j++) {
              g.d[j] = (int) input[j];}}
          break;
          
          /* Currently, only type double is supported for sparse graph 
           * array inputs. If other types are required, additional cases 
           * may be added here. */
          
      default :  
          mexErrMsgTxt("Sparse graph arrays must be type double.");                          
  }
  switch(mxGetClassID(prhs[1]))
  {
      case mxDOUBLE_CLASS : {
          double *input;
          input = (double *) mxGetData(prhs[1]);
          for (j = 0; j < nv; j++) {
              g.v[j] = (int) input[j] - 1;}}
          break;
          
          /* Currently, only type double is supported for sparse graph 
           * array inputs. If other types are required, additional cases 
           * may be added here. */
          
      default :  
          mexErrMsgTxt("Sparse graph arrays must be type double.");                          
  }
  switch(mxGetClassID(prhs[2]))
  {
      case mxDOUBLE_CLASS : {
          double *input;
          input = (double *) mxGetData(prhs[2]);
          for (j = 0; j < ne; j++) {
              g.e[j] = (int) input[j] - 1;}}
          break;
          
          /* Currently, only type double is supported for sparse graph 
           * array inputs. If other types are required, additional cases 
           * may be added here. */
          
      default :  
          mexErrMsgTxt("Sparse graph arrays must be type double.");                          
  }
  
  if (nrhs == 4)
  {
      switch(mxGetClassID(prhs[3]))
      {
          case mxDOUBLE_CLASS : {
              double *input;
              input = (double *) mxGetData(prhs[3]);
              for (j = 0; j < nv; j++) {
                  lab[j] = (int) input[j*2] - 1;
                  ptn[j] = (int) input[j*2 + 1]; }}
              break;
              
          /* Currently, only type double is supported for the coloring 
           * matrix. If other types are required, additional cases may
           * be added here. */    

          default :
           mexErrMsgTxt("Coloring matrix must be of type double.");                          
      }
      if (!setcmp((void *)lab, nv))
          mexErrMsgTxt("Illegal coloring matrix. See help.");
  }
  
  nauty_check(WORDSIZE,m,nv,NAUTYVERSIONID); /* check if linking to 
                                             * compatible versions of
                                             * Nauty routines */
  /* call the Nauty routine */
  /* note: This program does not allow active cells to be set and uses
   * the default (all cells active). Therefore, active is set to NULL. */
  nauty((graph *)&g,lab,ptn,NULL,orbits,&options,&stats,
									workspace,5*m,m,nv,(graph *)&canong);
  numgens = stats.numgenerators;
  Gsize = stats.grpsize1*pow(10.0,(double)stats.grpsize2);
  sortlists_sg(&canong); /* sorts neighborhood lists in ascending order */
  
  /* pack output into mxArrays */
  /* note: mxArray outputs always return type double matrices in this 
   * version. If other data types are required, output may be cast to the 
   * required type in the MATLAB command window using MATLAB internal 
   * functions. */
  
  plhs[0] = mxCreateDoubleMatrix(numgens, nv, mxREAL); /* S */
  plhs[1] = mxCreateDoubleMatrix(1, numgens, mxREAL);  /* vstab */
  plhs[2] = mxCreateDoubleScalar(Gsize);               /* Gsize */
  plhs[3] = mxCreateDoubleMatrix(1, nv, mxREAL);       /* orb */
  plhs[4] = mxCreateDoubleMatrix(1, nv, mxREAL);       /* lab */
  plhs[5] = mxCreateDoubleMatrix(1, nv, mxREAL);       /* canong.d */
  plhs[6] = mxCreateDoubleMatrix(1, nv, mxREAL);       /* canong.v */
  plhs[7] = mxCreateDoubleMatrix(1, ne, mxREAL);       /* canong.e */
  
  output[0] = mxGetPr(plhs[0]);
  output[1] = mxGetPr(plhs[1]);
  output[2] = mxGetPr(plhs[3]);
  output[3] = mxGetPr(plhs[4]);
  output[4] = mxGetPr(plhs[5]);
  output[5] = mxGetPr(plhs[6]);
  output[6] = mxGetPr(plhs[7]);
  
  for (i = 0; i < numgens; i++) {
      output[1][i] = (double) stabverts[i] + 1.0;
      for (j = 0; j < nv; j++) {
          output[0][j*numgens + i] = (double) generators[i*nv + j] + 1.0; }}
  
  for (j = 0; j < nv; j++) {
      output[2][j] = (double) orbits[j] + 1.0;
      output[3][j] = (double) lab[j] + 1.0;
      output[4][j] = (double) canong.d[j];
      output[5][j] = (double) canong.v[j] + 1.0; }
  
  for (j = 0; j < ne; j++) {
      output[6][j] = (double) canong.e[j] + 1.0; }
  
  /* clean-up */
  mxFree(g.d);
  mxFree(g.v);
  mxFree(g.e);
  mxFree(canong.d);
  mxFree(canong.v);
  mxFree(canong.e);
  mxFree(lab);
  mxFree(ptn);
  mxFree(orbits);
  mxFree(workspace);
  mxFree(generators);
  mxFree(stabverts);
}

/*=======================================================================*/
/* User defined procedures */
void GetGenerators(int count, permutation *perm, int *orbits, 
                                  int numorbits, int stabvertex, int n)
{
    int i;
    int *tmpgens;
    int *tmpstabs;
    
    tmpgens = (int *)mxRealloc(generators, count * n * sizeof(int));
    tmpstabs = (int *)mxRealloc(stabverts, count*sizeof(int));
    
    if (tmpgens != NULL && tmpstabs != NULL) {
        generators = tmpgens;
        stabverts = tmpstabs; }
    else {
        mxFree(generators);
        mxFree(stabverts);
        mexErrMsgTxt("mxRealloc failed."); }

    for (i = 0; i < n; i++) {
        generators[(count - 1) * n + i] = (int)perm[i]; }
    stabverts[(count - 1)] = stabvertex;
}

int setcmp(void *set, int num)
{
    void *sortset;
    int i, flag;
    int cmpels(const void *, const void *);
    
    sortset = mxMalloc(num*sizeof(int));
    if (sortset == NULL)
      mexErrMsgTxt("mxMalloc failed.");
    memcpy(sortset, set, num*sizeof(int));
    
    qsort(sortset, num, sizeof(int), cmpels);
    
    flag = 1;
    for (i = 0; i < num; i++) {
        if ( ((int *)sortset)[i] != i ) {
            flag = 0; 
            break; }}
    
    mxFree(sortset);
    return(flag);
}

int cmpels(const void *arg1, const void *arg2)
{
    return( *(int *)arg1 - *(int *)arg2 );
}