function [f] = Decode(fcode,finit,T,Tsize,Trelab,numdigits,startofdigit,endofdigit)
%DECODE decodes the encoded representation of a function fcode back to 
%its standard representation f.
%
% INPUT:  fcode is a 1 by numdigits row vector which is the encoded version
%         of a function f.
%
%         finit is the function which was used to generate an encoding
%         scheme using the function GenerateEncodingScheme.
%
%         T is a row vector representing the partition {S1,S2,...,Sr} of 
%         the set S of component states such that the value of T(s) is the 
%         same for all s in Si and different for all s not in Si. Si
%         contains the set of possible states for a component of type i.
%
%         Tsize is a row vector representing a map from each state s in S 
%         to a component size, which is the number of states connected to
%         s (i.e. the cardinality of the subset Si containing s). The
%         coding scheme regards Tsize(f(i)) for each i as the base of the 
%         ith digit of f with f representing a number in an arbitrary 
%         number system.
%
%         Trelab is a row vector representing a function which relabels 
%         each state s belonging to Si such that the set of labels are 
%         {0,1,...,Tsize(s)-1}.
%
%         numdigits is the integer number of double precision digits
%         (decimal numbers) required to encode a function f.
%
%         startofdigit is a 1 by numdigits row vector which is an index 
%         into the first digit of f belonging to an encoded decimal digit 
%         such that startdigit(i) is the first digit of f belonging to the 
%         ith decimal digit.
%
%         endofdigit is a 1 by numdigits row vector which is an index into 
%         the last digit of f belonging to an encoded decimal digit such 
%         that startdigit(i) is the last digit of f belonging to the ith
%         decimal digit.
%
% OUTPUT: f is a size(finit) row vector which is the decoded version of
%         fcode.
%
% See also, Encode, GenerateCodingScheme, IDComponentTypes
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011

p = length(finit);
f = zeros(1,p);
for i = 1:numdigits
    bprod = prod(Tsize(finit(startofdigit(i)+1:endofdigit(i))));
    numerator = fcode(i);
    f(startofdigit(i)) = floor(numerator/bprod);
    for j = startofdigit(i)+1:endofdigit(i)
        numerator = numerator - f(j-1)*bprod;
        bprod = bprod/Tsize(finit(j));
        f(j) = floor(numerator/bprod);
    end
end
for i = 1:p
    f(i) = find(Trelab == f(i) & T == T(finit(i)));
end
end