function [Vsubs,Gtildesubs,N] = FindUniqueSubsystems(pi,T,Gtilde,finit)
%FINDUNIQUESUBSETS finds unique (non-isomorphic) subsystems among the set
%of all subsystems.
%
% INPUT:  pi is a numeric row vector representing the partition 
%         {D1,D2,...,Dn} of domain places such that all d in Di have the 
%         same value pi(d), and all d not in Di have a different value 
%         pi(d).
%
%         T is a row vector representing the partition {S1,S2,...,Sr} of 
%         the set S of component states such that the value of T(s) is the 
%         same for all s in Si and different for all s not in Si. Si
%         contains the set of possible states for a component of type i.
%
%         Gtilde is a vertex-colored equivalent of a characteristic graph G
%         which is an output of the function GenerateCharacteristicGraph. 
%         (See GenerateCharacteristicGraph help for more details).
%
%         finit is a row vector representing a finite-valued function which 
%         is the root state in the system reachability graph.
%
% OUTPUT: Vsubs is an m by 1 cell array, where m is the number of unique
%         subsystems. Each cell j holds a subset Di in the partition
%         {D1,D2,...,Dn}, representing the domain places belonging to the
%         jth unique subsystem.
%
%         Gtildesubs is an m by 1 cell array where cell j holds the
%         characteristic graph of subsystem j, defined as the induced
%         subgraph of Gtilde containing vertex set Vsubs{j}.
%
%         N is an m by 1 numeric vector where N(j) represents the number of
%         subsystems that are isomorphic to subsystem j in the full system.
%
% notes: This function calls the functions SetupNautyGraph_Sparse and 
% CallNauty_sparse.
%
% See also, FindIndependentSubsets, MergeSubsets
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Bugenhagen, SM et al. Manuscript in preparation.     

Tsubs = unique(pi);
psubs = length(Tsubs);
Vsubs = cell(psubs,1);
Gtildesubs = cell(psubs,2);
L = Gtilde{1,1};
Utilde = Gtilde{1,2};
for j = 1:psubs
    Vsubs{j} = find(pi == Tsubs(j));
    ixUtilde = false(size(Utilde,1),1);
    for i = 1:length(ixUtilde)
        if any(ismember(Vsubs{j},Utilde(i,:)))
            ixUtilde(i) = true;
        end
    end
    Gtildesubs{j,2} = Utilde(ixUtilde,:);
    Vtilde = sort(union(Vsubs{j},unique(Gtildesubs{j,2})));
    Gtildesubs{j,1} = L(Vtilde);
    for i = 1:length(Vtilde)
        ixUtildesubs = Gtildesubs{j,2} == Vtilde(i);
        Gtildesubs{j,2}(ixUtildesubs) = i;
    end
end
pisubs = 1:psubs;
unprocessed = 1:psubs;
while any(unprocessed)
    source = unprocessed(find(unprocessed,1,'first'));
    unprocessed(unprocessed == source) = 0;
    for j = find(unprocessed);
        if isequal(sort(T(finit(Vsubs{j}))),sort(T(finit(Vsubs{source}))))
            if isequal(sort(Gtildesubs{j,1}),sort(Gtildesubs{source,1}))
                if isempty(Gtildesubs{j,2}) && isempty(Gtildesubs{source,2})
                    tf = true;
                else
                    [d1,v1,e1,c1] = ...
                              SetupNautyGraph_sparse(Gtildesubs(source,:));
                    [d2,v2,e2,c2] = ...
                                   SetupNautyGraph_sparse(Gtildesubs(j,:));
                    [~,~,~,~,~,dcanon1,vcanon1,ecanon1] = ...
                                             CallNauty_sparse(d1,v1,e1,c1);
                    [~,~,~,~,~,dcanon2,vcanon2,ecanon2] = ...
                                             CallNauty_sparse(d2,v2,e2,c2);
                    tf = isequal(dcanon1,dcanon2) && ...
                         isequal(vcanon1,vcanon2) && ...
                         isequal(ecanon1,ecanon2);
                end
                if tf
                    pisubs(j) = source;
                    unprocessed(j) = 0;
                end
            end
        end
    end
end
[subsix, ends] = unique(sort(pisubs),'last');
N = zeros(length(subsix),1);
N(1) = ends(1);
N(2:end) = ends(2:end) - ends(1:end-1);
Vsubs = Vsubs(subsix,1);
Gtildesubs = Gtildesubs(subsix,:);
end