function [G,Gprime,Gtilde] = GenerateCharacteristicGraph(finit,T,rules,D)
%GENERATECHARACTERISTICGRAPH Generates the system characteristic graph from
%a set of dependency matrices.
%
% INPUT:  finit is a row vector representing a finite-valued function which 
%         is the root state in the system reachability graph.
%
%         T is a row vector representing the partition {S1,S2,...,Sr} of 
%         the set S of component states such that the value of T(s) is the 
%         same for all s in Si and different for all s not in Si. 
%
%         rules is a 4 column numeric array representing the transition 
%         rules. The first column contains indices into the set Q of 
%         component transitions, the second column contains indices into 
%         the set C of rate constants, the third column contains indices 
%         into the set D of dependency matrices, and the fourth column 
%         contains a flag indicating the format of its associated 
%         dependency matrix (zero indicates spatial coupling format, a 
%         nonzero indicates mean field coupling format). Each row j is a 
%         mapping from the component transition Q(rules(j,1)) to the rate 
%         constant C(rules(j,2)) under the restriction that the system
%         state satisfies dependency matrix D(rules(j,3)). If a rate 
%         constant cj in C is not associated with a dependency graph (i.e. 
%         has no functional dependencies), then rules(j,3) should be set to 
%         zero (rules(j,4) is ignored in this case).
%
%         D is a cell vector of dependency matrices. A dependency matrix in
%         spatial coupling format is given as a q by 3 cell array (where q 
%         is the number of hyperarcs in the dependency graph) with the 
%         first column containing the tail sets Ay, the second containing 
%         the head sets By, and the third containing the coloring function 
%         Ry of each yth hyperarc such that rho{Ry(j)} is the color of 
%         By(j). A dependency matrix in mean field coupling format is given
%         as a 2 row matrix with the first row containing indices into rho,
%         and the second containing an integer value such that D{i}(2,j) is
%         the number of components required to be in the state class 
%         D{i}(1,j).
%
% OUTPUT: G is a hypergraph containing vertices V and hyperarcs U, 
%         representing the characteristic graph of the system. It is
%         given as a 1 by 2 cell array, with the first cell containing a 
%         numeric vector representing a labeling function L of the vertices
%         V, and the second cell containing a cell array representing the 
%         hyperarcs U. The vertices are labeled according to their 
%         component type. Each row j in U represents a hyperarc in the
%         characteristic graph, with column 1 containing the tail sets,
%         column 2 the head sets, column 3 the component state coloring
%         functions, and column 4 the rate constant colors.
%
%         Gprime is an arc-colored bipartite equivalent of the hypergraph G
%         with vertices Vprime and arcs Uprime. It given as a 1 by 3 cell 
%         array with the first cell containing a numeric vector
%         representing a labeling function Lprime of the vertices Vprime, 
%         the second cell containing a numeric array with rows representing
%         simple arcs (corresponding to the tails of the hyperarcs in G)
%         with tails in column 1 and heads in column 2, and the third cell
%         containing a numeric array with rows representing colored arcs
%         (corresponding to the heads of the hyperarcs in G) with tails in
%         column 1, heads in column 2, component state colors in column 3,
%         and rate constant colors in column 4. The vertices in Vprime\V 
%         are labeled with a zero, and the remaining vertices are labeled
%         as in V.
%
%         Gtilde is a vertex-colored equivalent of Gprime with vertices 
%         Vtilde and arcs Utilde, resulting from the subdivision of the 
%         colored arcs in (cell 3 of) Gprime. It is given as a 1 by 2 cell 
%         array with the first cell containing a numeric vector 
%         representing a labeling function Ltilde of the vertices, and the 
%         second cell containing a numeric array with rows representing 
%         simple arcs with tails in column 1 and heads in column 2. 
%         Vertices are labeled such that the vertices in Vtilde\Vprime are 
%         labeled with positive integer values, and the remaining vertices
%         are labeled as in Vprime.
%
% See also, CallNauty_sparse.c, CallNauty_packed.c, SetupNauty_sparse,
% SetupNauty_packed
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     10/24/11
%
% References:
% 1. Bugenhagen, SM et al. Manuscript in preparation.

%% Check for dependencies
p = length(finit);
V = zeros(p,1);
V(1:p) = -T(finit);
if isempty(D) || ~any(rules(:,3)) || all(rules(logical(rules(:,3)),4))
    U = [];
    G = {V,U};
    Gprime = {V,U};
    Gtilde = {V,U};
    return;
end

%% Generate G
rulesix = find(rules(:,3) & ~rules(:,4))';
q = 0;
for i=rulesix
    q = q + size(D{rules(i,3)},1);      
end
U = cell(q,4);
k = 0;
for i=rulesix
    Di = D{rules(i,3)};
    for j=1:size(Di,1);
        k = k + 1;
        U(k,1:3) = Di(j,:);
        U{k,4} = rules(i,2);
    end         
end
G = {V,U};

%% Generate Gprime
L = zeros(1,q);
unprocessed = 1:q;
while any(unprocessed)
    source = unprocessed(find(unprocessed,1,'first'));
    L(source) = source;
    unprocessed(source) = 0;
    for i=find(unprocessed)
        if isequal(U(source,1),U(i,1)) && isequal(U(source,2),U(i,2))
            L(i) = source;
            unprocessed(i) = 0;
        end
    end
end
[L,ix] = sort(L);
U = U(ix,:);
[~,ixlast] = unique(L);
Rprime = cell(length(ixlast),1);
nheads = 0;
for i=1:length(ixlast)
    headcard = length(U{ixlast(i),3});
    Rprime{i} = zeros(1,headcard);
    nheads = nheads + headcard;
end
unprocessed = 1:nheads;
while any(unprocessed)
    source = unprocessed(find(unprocessed,1,'first'));
    unprocessed(source) = 0;
    x = 0;
    for i=1:length(ixlast)
        for j=1:length(Rprime{i})
            x = x + 1;
            if x == source
                break
            end
        end
        if x == source
            break
        end
    end
    if i == 1
        Uix = 1:ixlast(1);
    else
        Uix = ixlast(i-1)+1:ixlast(i);
    end
    Ri = cell2mat(U(Uix,3:4));
    sourceset = [Ri(:,j),Ri(:,size(Ri,2))];
    vals = (max(sourceset(:,2))+1)*sourceset(:,1) + ...
            sourceset(:,2);
    [~,ixsort] = sort(vals);
    sourceset = sourceset(ixsort,:);
    Rprime{i}(j) = source;
    x = 0;
    for i=1:length(ixlast)
        if i == 1
            Uix = 1:ixlast(1);
        else
            Uix = ixlast(i-1)+1:ixlast(i);
        end
        Ri = cell2mat(U(Uix,3:4));
        for j=1:length(Rprime{i})
            x = x + 1;
            if unprocessed(x)
                xset = [Ri(:,j),Ri(:,size(Ri,2))];
                vals = (max(xset(:,2))+1)*xset(:,1) + xset(:,2);
                [~,ixsort] = sort(vals);
                xset = xset(ixsort,:);
                if isequal(xset,sourceset)
                    Rprime{i}(j) = source;
                    unprocessed(x) = 0;
                end
            end
        end
    end
end
Uprime = [U(ixlast,1:2),Rprime];
Vprime = V;
Gprime = {Vprime,Uprime};

%% Generate Gprime2
q = size(Uprime,1);
rt = 0;
rh = 0;
for i=1:q
    lenA = length(Uprime{i,1});
    lenB = length(Uprime{i,2});
    if lenA ~= 1 || lenB ~= 1
        rt = rt + lenA;
    end
    rh = rh + lenB;
end
Uprime2t = zeros(rt,2);
Uprime2h = zeros(rh,3);
m = p;
nt = 0;
nh = 0;
for i=1:q    
    A = Uprime{i,1};
    B = Uprime{i,2};
    R = Uprime{i,3};
    lenA = length(A);
    lenB = length(B);
    if lenA ~= 1 || lenB ~=1
        m = m + 1;
        for j=1:lenA
            nt = nt + 1;
            Uprime2t(nt,1) = A(j);
            Uprime2t(nt,2) = m;
        end
        for j=1:lenB
            nh = nh + 1;
            Uprime2h(nh,1) = m;
            Uprime2h(nh,2) = B(j);
            Uprime2h(nh,3) = R(j);
        end
    else
        nh = nh + 1;
        Uprime2h(nh,1) = A;
        Uprime2h(nh,2) = B;
        Uprime2h(nh,3) = R;
    end
end
Vprime2 = zeros(m,1);
Vprime2(1:p) = -T(finit);
Vprime2(p+1:m) = 0;
%Gprime2 = {Vprime2, Uprime2t, Uprime2h};

%% Generate Gtilde
Vtilde = Uprime2h(:,3);
Vtildemaxn = 0;
for b = unique(Vtilde');
    if nnz(Vtilde == b) > Vtildemaxn
        Vtildemc = b;
        Vtildemaxn = nnz(Vtilde == b);
    end
end
ixv = find(Vtilde ~= Vtildemc);
ixr = find(Vtilde == Vtildemc);
Vtilde = Vtilde(Vtilde ~= Vtildemc);
Utilde = zeros(2*length(ixv)+length(ixr),2);
for i=1:length(ixv)
    Utilde(2*i-1,1) = Uprime2h(ixv(i),1);
    Utilde(2*i-1,2) = m+i;
    Utilde(2*i,1) = m+i;
    Utilde(2*i,2) = Uprime2h(ixv(i),2);
end
for i=1:length(ixr)
    Utilde(2*length(ixv)+i,1) = Uprime2h(ixr(i),1);
    Utilde(2*length(ixv)+i,2) = Uprime2h(ixr(i),2);
end
Vtilde = [Vprime2; Vtilde];
Utilde = [Uprime2t; Utilde];
Gtilde = {Vtilde, Utilde};

end