function [pi] = MergeSubsets(Q,pi,T,finit)
%MERGESUBSETS updates the partition pi, computed by the
%FindIndependentSubsets function, by merging together subsets of vertices
%when synchronizing transitions involving components of disjoint subsets
%are present in the set of component transitions Q.
%
% INPUT:  Q is a cell array representing the set of component transitions
%         such that Q{j,1} represents the source states Xj, Q{j,2}
%         represents the mapping Yj from Xj to the destinations states in 
%         S, and Q{j,3} represents the multiplicity function Zj.
%
%         pi is a partition of the vertices of the system characteristic 
%         graph into maximally connected subsets such that the value pi(i) 
%         is the same for each ith vertex belonging to a given maximally 
%         connected subset and different for each vertex not belonging the 
%         subset.
%
%         T is a row vector representing the partition {S1,S2,...,Sr} of 
%         the set S of component states such that the value of T(s) is the 
%         same for all s in Si and different for all s not in Si. Si
%         contains the set of reachable states for a component of type i.
%
%         finit is a row vector representing a finite-valued function which 
%         is the root state in the system reachability graph.
%
% OUTPUT: pi is the updated partition.
%
% notes: This function calls the function multisetdiff.
%
% See also, FindIndependentSubsets, FindUniqueSubsets
%             
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Bugenhagen, SM et al. Manuscript in preparation.

p = length(finit);
V = 1:p;
unprocessed = V;
qix = find(~cellfun(@(x)isequal(x,1),Q(:,3)))';
while any(unprocessed)
    if length(unique(pi)) == 1
        return;
    end
    v = unprocessed(find(unprocessed,1,'first'));
    unprocessed(unprocessed==v) = 0;
    for j = qix
        X = Q{j,1};
        Z = Q{j,3};
        s = X(find(T(X) == T(finit(v)),1));
        if ~isempty(s)
            for d = setdiff(unprocessed,0)
                if pi(d) ~= pi(v)
                    if any(T(multisetdiff(X,Z,s,1)) == T(finit(d)))
                        dp = V(pi == pi(d));
                        pi(dp) = pi(v);
                    end
                end
            end
        end
    end
end
end