function [Fprime,Tprime] = ReorderF(F,T,fcn)
%REORDERF Changes the order in which the system states are listed in F
%using the permutation returned by a function fcn.
%
% INPUT:  F is an array of system states (each row is a system state) 
%         representing a set, or a set of equivalence classes of, system 
%         states.
%         
%         T is an array of transitions between system states in F such that
%         (T(i,1),T(i,1)) is the transition from state F(T(i,1)) to state 
%         F(T(i,2)).
%
%         fcn is a function handle to a reordering function of a sparse
%         matrix. It may be a handle to a user defined function, or to one
%         of the built-in MATLAB functions: colperm; colamd; symamd; amd; 
%         or symrcm.
%
% OUTPUT: Fprime is the array F of system states reordered according to the
%         permutation returned by the function fcn.
%
%         Tprime is the array T of system state transitions with elements
%         updated to account for the reordering of F.
%
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References:
% 1. "Permutations and Reordering." MATLAB User's Guide.

n = length(F);
S = sparse(double(T(:,2)),double(T(:,1)),ones(size(T,1),1),n,n);
tf = isa(fcn, 'function_handle');
if ~tf
    error('fcn must be a function handle');
end
p = fcn(S);
pinv(p) = 1:n;
Tprime = pinv(T);
Fprime = F(p,:);
end