function [P] = SolveMasterEqtn_SS(F,T,W,M,C)
%SOLVEMASTEREQTN_SS sets up and computes the steady-state solution
%to the master equation associated with a system's state-space graph using
%the MATLAB backslash operator.
%
% INPUT:  F is an array of system states (each row is a system state) 
%         representing a set, or a set of equivalence classes of, system 
%         states.
%         
%         T is an array of transitions between system states in F such that
%         (T(i,1),T(i,1)) is the transition from state F(T(i,1)) to state 
%         F(T(i,2)).
%
%         W is a size(T,1) vector representing the weighting function of T
%         such that W(i) is the weight associated with transition T(i,:). 
%         A weight W(i) is an index into the set C of rate constants.
%
%         M is a size(T,1) vector representing a multiplicity function of T
%         such that M(i) is the integer-valued multiplicity associated with 
%         transition T(i,:). In the case that the multiplicity function's 
%         range is singleton, M may be given as a scalar value.
%
%         C is a numeric vector representing a set of specific probability
%         rate constants.
%
% OUTPUT: P is the steady-state solution to the master equation such that
%         P(i,:) is the stationary probability of the system occupying 
%         state F(i).
%
% See also, SolveMasterEqtn_transient
% 
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011

T = double(T);
M = double(M);
C = double(C(:));
n = length(F);
m = size(T,1);
i = [T(:,2); (1:n)'; n.*ones(n,1)];
j = [T(:,1); (1:n)'; (1:n)'];
s = zeros(m+2*n,1);
s(1:m) = M.*C(W);
A = sparse(i,j,s,n,n);
s(m+1:m+n) = -sum(A,1);
s(i==n) = 0;
s(m+n+1:end) = 1;
A = sparse(i,j,s,n,n);
P = A\[zeros(n-1,1); 1];
end