function [nprime,Qprime,rulesprime,Cprime] = SystemToComponent(F,T,W,M,Cmap,Dmap)
%SYSTEMTOCOMPONENT generates a new component graph from a system-state
%graph along with a rules structure for a model composed of the new
%component(s). It is useful for aggregating smaller components into larger 
%ones when constructing large compositional models.
%
% INPUT:  F, T, W, and M are outputs from the Vernan tool representing a
%         system state graph (see help Vernan for more details). 
%
%         Cmap is a numeric cell vector representing a map from the set C 
%         of rate constants used to construct (F,T,W,M) to a set Ctilde of 
%         new rate constants such that C(i) maps to the subset
%         Ctilde(Cmap{i}) of elements in Ctilde.
%
%         Dmap is a two column numeric array representing a map from the 
%         set Ctilde to a set Dtilde of new dependency matrices such that 
%         Ctilde(i) maps to Dtilde{Dmap(i,1)}, where Dtilde{Dmap(i,1)} is a 
%         dependency matrix in spatial coupling format if Dmap(i,2) is
%         zero, and is a dependency matrix in mean-field coupling format if
%         Dmap(i,2) is non-zero. If Ctilde(i) is not associated with a
%         dependency matrix (i.e. is associated with a constant
%         transition), then Dmap(i,1) should be set to zero (in this case, 
%         Dmap(i,2) is ignored).
%
% OUTPUT: nprime is a scalar number representing the total number of 
%         possible component states in the updated component graph.
%
%         Qprime is a 3-column cell array representing the set of component 
%         transitions in the updated component graph (see help Vernan for 
%         more details).
%
%         rulesprime is a 4-column numeric array representing the
%         transition rules for the new system as specified by Cmap and
%         Dmap (see help Vernan for more details).
%
%         Cprime is a size(rulesprime,1) by 2 numeric array used to compute
%         the set of specific probability rate constants associated with
%         the component transitions in Qprime. Column 1 contains values
%         from the set M, and column 2 contains indices into the set Ctilde
%         of rate constants. The ith rate constant in Cprime is calculated 
%         by multiplying Cprime(i,1) by Ctilde(Cprime(i,2)).
%
% Notes: The new component graph generated by SystemToComponent does not
%        contain any synchronizing transitions. If synchronizing
%        transitions are required, the output would need to be manually 
%        edited.
% 
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011

T = double(T);
M = double(M);
if isscalar(M)
    M = M*ones(size(W));
end
nprime = size(F,1);
Qprime = cell(size(T,1),3);
ixCprime = cell(size(T,1),1);
l = 0;
for j = 1:size(Qprime,1)
    Qprime{j,1} = T(j,1);
    Qprime{j,2} = T(j,2);
    Qprime{j,3} = 1;
    lenCprimej = length(Cmap{W(j)});
    ixCprime{j} = zeros(1,lenCprimej);
    for i = 1:lenCprimej
        l = l+1;
        ixCprime{j}(i) = l;
    end
end
rulesprime = zeros(l,4);
Cprime = zeros(l,2);
l = 0;
for j = 1:size(Qprime,1)
    lenCprimej = length(ixCprime{j});
    for i = 1:lenCprimej
        l = l+1;
        rulesprime(l,1) = j;
        rulesprime(l,2) = l;
        ixCtilde = Cmap{W(j)}(i);
        rulesprime(l,3) = Dmap(ixCtilde,1);
        rulesprime(l,4) = Dmap(ixCtilde,2);
        Cprime(l,1) = M(j);
        Cprime(l,2) = ixCtilde;
    end
end    
end