function [F,T,W,M,S,Vsubs,N] = Vernan(n,Q,rules,D,rho,finit,symred,invmanid,decodeout)
%VERNAN is a tool for automatically constructing reduced-order models of
%discrete-state processes
%
% DESCRIPTION:
% Vernan, named after Vernanimalcula guizhouena (the earliest known animal
% exhibiting bilateral symmetry), is a tool for automatically computing and 
% minimizing the state-space of finite state transition systems 
% (see ref [1]). It does this by identifying symmetries (using the Nauty 
% routine [2]) at the system description level and accounting for these 
% symmetries when computing the reachable state-space of the system [1,3]. 
% Vernan can also be used to identify independence among components of a 
% system. Independence of system components or groups of system components 
% can be exploited to achieve a further reduction in the dimension of the 
% system master equation [4].
%
% INPUT:  n is an integer value representing the size of the set S of
%         possible component states.
%
%         Q is a cell array representing the set of component transitions
%         such that Q{j,1} represents the source states Xj, Q{j,2}
%         represents the mapping Yj from Xj to the destination states in 
%         S, and Q{j,3} represents the multiplicity function Zj. 
%         Specifically, a transition j involves Q{j,3}(i) components in 
%         state Q{j,1}(i), each of which map to state Q{j,2}(i), for 
%         i = 1,...,length(Q{j,1}).
%
%         rules is a 4 column numeric array representing the transition 
%         rules. The first column contains indices into the set Q of 
%         component transitions, the second column contains indices into 
%         the set C of rate constants, the third column contains indices 
%         into the set D of dependency matrices, and the fourth column 
%         contains a flag indicating the format of its associated 
%         dependency matrix (zero indicates spatial coupling format, a 
%         nonzero indicates mean field coupling format). Each row j is a 
%         mapping from the component transition Q(rules(j,1)) to the rate 
%         constant C(rules(j,2)) under the restriction that the system
%         state satisfies dependency matrix D(rules(j,3)). If a rate 
%         constant cj in C is not associated with a dependency matrix (i.e. 
%         has no functional dependencies), then rules(j,3) should be set to 
%         zero (rules(j,4) is ignored in this case).
%
%         D is a cell vector of dependency matrices. A dependency matrix in
%         spatial coupling format is given as a q by 3 cell array (where q 
%         is the number of hyperarcs in the dependency graph) with the 
%         first column containing the tail sets Ay, the second containing 
%         the head sets By, and the third containing the coloring function 
%         Ry of each yth hyperarc such that rho{Ry(j)} is the color of 
%         By(j). A dependency matrix in mean field coupling format is given
%         as a 2 row matrix with the first row containing indices into rho,
%         and the second containing an integer value such that D{i}(2,j) is
%         the number of components required to be in the state class 
%         D{i}(1,j).
%
%         rho is cell vector of component state classes. Each cell contains
%         a unique subset of the set S of component states.
%
%         finit is a row vector representing a finite-valued function from 
%         the component places to S, which is the root state in the system 
%         reachability graph.
%
%         symred if a flag indicating whether or not to apply symmetry
%         reduction when constructing the model. If symred is set to 'on',
%         symmetry reduction is applied (this is the default), otherwise
%         set symred to 'off' to generate the full model.
%
%         invmanid is a flag indicating whether or not to check for an
%         invariant manifold in the system. If set to 'on', Vernan 
%         checks whether any subset of system components is independent 
%         from other components. Set invmanid to 'off' (the default) to 
%         bypass this test. If invmanid is set to 'on' and an invariant 
%         manifold is detected, Vernan returns subsets Vsubs of system
%         components which are independent from one another along with the
%         numbers N of each of the independent subsets in the full system.
%         Note that the current version of Vernan does not itself use this
%         information to reduce the state space. Instead, it is intended to
%         aid the user in reformulating the inputs so that Vernan can be
%         applied to each independent component subset separately.
%
%         decodeout is a flag indicating whether or not to decode the
%         output array F. Vernan applies a hash function to the set of
%         system states in F for efficiency purposes, and if decodeout is 
%         set to 'off', F is returned as a set of hash values. By default,
%         decodeout is set to 'on', and the actual functions corresponding 
%         to system states are returned. Note that the Vernan routine 
%         Decode may be applied to an individual hash value to obtain 
%         its associated function.
%
% OUTPUT: F is an array of system states (each row is a system state) 
%         representing the set of system states reachable from finit. If 
%         there exists symmetry in the system and symred is set to 'on', 
%         then the set of system states returned is actually a set of 
%         representatives from the equivalence classes induced by the 
%         system symmetry group acting on the system states. That is, F is 
%         a transversal of the patterns of the full state-space. Otherwise,
%         F is the full state-space. If decodeout is set to 'off', the 
%         system states in F are given as hash values, otherwise decodeout 
%         is set to 'on' and system states are given as functions from the 
%         domain places to the set S of system states.
%
%         T is an array of transitions between system states in F such that
%         (T(i,1),T(i,1)) is the transition from state F(T(i,1)) to state 
%         F(T(i,2)).
%
%         W is a size(T,1) vector representing the weighting function of T
%         such that W(i) is the weight associated with transition T(i,:). 
%         Here, a weight W(i) is an index (in the set rules(:,2)) into a 
%         set of rate constants.
%
%         M is a size(T,1) vector representing a multiplicity function of T
%         such that M(i) is the integer-valued multiplicity associated with 
%         transition T(i,:). In the case that the symmetry group of the
%         system is the identity or symred is set to 'off', the 
%         multiplicity function is the trivial function with the range {1},
%         in which case M is returned as an empty vector.
%
%         S is an array of generators of the system symmetry group. Each
%         row is a generating permutation of the set {1,...,length(finit)}.
%         If symred is set to 'off', S is returned as an empty array.
%
%         Vsubs is an m by 1 cell array, where m is the number of unique
%         subsystems. Each cell j holds an independent subset of component 
%         places representing the components belonging to the jth unique 
%         subsystem. If invmanid is set to 'off', Vsubs is returned as an
%         emtpy array.
%
%         N is an m by 1 numeric vector where N(j) represents the number of
%         subsystems that are equivalent (isomorphic) to subsystem j in the
%         full system. If invmanid is set to 'off', N is returned as an
%         empty vector.
%
% Notes: This function calls the following functions:
%        IDComponentTypes
%        GenerateCodingScheme
%        GenerateCharacteristicGraph
%        FindIndependentSubsets
%        MergeSubsets
%        FindUniqueSubsystems
%        SetupNautyGraph_sparse
%        CallNauty_sparse
%        SchreierSims
%        GenerateFullStateSpaceGraph
%        GenerateReducedStateSpaceGraph
%        DecodeAll
%        
% Author:   Scott Bugenhagen
% Contact:  sbugenha@mcw.edu
% Date:     8/30/2011
%
% References: 
% 1. Bugenhagen, SM et al. Manuscript in preparation.
% 2. McKay, BD. Nauty user's guide (version 2.4). Australian National 
%    University, 2009. Retrieved from <http://cs.anu.edu.au/~bdm/nauty/> 
%    on 6/1/2011.
% 3. Junttila T. On the symmetry reduction method for petri nets and 
%    similar formalisms (doctoral thesis). Helsinki, Finland: Helsinki 
%    University of Technology, 2003.
% 4. Keener JP. Invariant manifold reductions for Markovian ion channel
%    dynamics. J Math Biol. 2009 Mar;58(3):447-57.

if nargin < 9
    decodeout = 'on';
end
if nargin < 8
    invmanid = 'off';
end
if nargin < 7
    symred = 'on';
end   
F = [];
T = [];
W = [];
M = [];
S = [];
Vsubs = [];
N = [];
p = length(finit);
for i = 1:length(rho)
    rho{i} = sort(rho{i});
end
for i = 1:length(D)
    Di = D{i};
    if iscell(Di)
        for j = 1:size(Di,1)
            Di{j,1} = sort(Di{j,1});
            [Di{j,2},ix] = sort(Di{j,2});
            Di{j,3} = Di{j,3}(ix);
        end
        D{i} = Di;
    end
end
[Tscript,Tsize,Trelab] = IDComponentTypes(n,Q);
[numdigits,startofdigit,endofdigit] = GenerateCodingScheme(finit,Tsize);
if strcmp(symred,'on') || strcmp(invmanid,'on')
    [G,~,Gtilde] = GenerateCharacteristicGraph(finit,Tscript,rules,D);
end
if strcmp(invmanid,'on')
    [pi] = FindIndependentSubsets(G{2},finit,Tscript,rules,D,rho);
    [pi] = MergeSubsets(Q,pi,Tscript,finit);
    [Vsubs,~,N] = FindUniqueSubsystems(pi,Tscript,Gtilde,finit);
    if ~isequal(N,1)
        prompt = 'Invariant manifold detected. Continue? Y/N [N]: ';
        reply = input(prompt,'s');
        if ~strcmpi(reply,'Y')
            disp('State-space not computed.');
            return
        end
    end
end
clear G;
if strcmp(symred,'on')
    [d,v,e,c] = SetupNautyGraph_sparse(Gtilde);
    [S,vstab,Gsize,~,~,~,~,~] = CallNauty_sparse(d,v,e,c);
    S = S(:,1:p);
    if Gsize == 1
        prompt = ['No symmetries detected. Compute full state-space? ', ...
                                                              'Y/N [N]: '];
        reply = input(prompt,'s');
        if ~strcmpi(reply,'Y')
            disp('State-space not computed.');
            return
        end
    end
    if Gsize ~= 1
        [base,~,~,U] = SchreierSims(S,vstab);
        [F,T,W,M] = GenerateReducedStateSpaceGraph(Q,rules,D,rho,finit, ...
                    Tscript,Tsize,Trelab,numdigits,startofdigit, ...
                                                endofdigit,d,v,e,c,base,U);
        if strcmp(decodeout,'on')
            F = DecodeAll(F,finit,Tscript,Tsize,Trelab,numdigits, ...
                                                  startofdigit,endofdigit);
        end
        return
    end
end
[F,T,W] = GenerateFullStateSpaceGraph(Q,rules,D,rho,finit,Tscript, ...
                           Tsize,Trelab,numdigits,startofdigit,endofdigit);
if strcmp(decodeout,'on')
    F = DecodeAll(F,finit,Tscript,Tsize,Trelab,numdigits, ...
                                                  startofdigit,endofdigit);
end
end